/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Mnenhy.
 *
 * The Initial Developer of the Original Code is Karsten Düsterloh <mnenhy@tprac.de>.
 *
 * Portions created by the Initial Developer are Copyright (C) 2002-2006
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

// +-
// |    Mnenhy - MailNews-Enhancements
// |
// |    Written, forged, copied and conceived by Karsten Düsterloh <mnenhy@tprac.de>
// +-
// |
// |    This XPI understands these http arguments, given via
// |
// |        http:///mnenhy.xpi?arguments
// |
// |    |arguments| must be the string representation of an array of objects,
// |    each object may define these members:
// |
// |        lang      the language code of the language to use during the install
// |                  (has to match one of the .properties file locations);
// |                  only the last |lang| argument will be used;
// |                  defaults to "en-US"
// |        locale    the language code of the language pack to install;
// |                  defaults to "all contained locales"
// |        content   the name of the mnenhy *sub*package to install;
// |                  (the base package will always be installed when installing content!);
// |                  defaults to "all contained subpackages"
// |
// |    Installing content can only be done if no Mnenhy installation exists yet!
// |    (This can be overridden by setting he argument |overwrite| to |true|.)
// |    The user will be prompted for profile or application install.
// |    If any content is installed, the base content will be installed, too.
// |
// |    Installing only locales can only be done if a Mnenhy installation already exists!
// |    The locale will then be installed into the content's location.
// |
// |    If neither content nor locale is given as arguments, all content and all locales
// |    will be installed.
// |
// |    Example: (on one line!)
// |
// |      http:///mnenhy.xpi?[{lang: "pl-PL", locale: "de-DE", content: "codecs"},
// |                          {lang: "de-DE", locale: "en-US", overwrite: true}]
// |
// |      will install the base content package, plus the codecs subpackage, plus the
// |      locales "de-DE" and "en-US". It will use "de-DE" for installation dialogs (because
// |      that |lang| argument came last) and will not check if Mnenhy is already installed.
// |
// +-

// |
// |  constants
// |
const gksName          = "Mnenhy - MailNews-Enhancements";
const gksVersion       = "0.7.6.0";  // major, minor, release, build > 0
const gksChromeName    = "mnenhy";
const gksContentDir    = "content/";
const gkoContentPacks  = {"mnenhy"     : ""            ,  // base package; must be named gksChromeName!
                          "headers"    : "headers/"    ,  // -+
                          "codecs"     : "codecs/"     ,  //  |
                          "chroman"    : "chroman/"    ,  //  +- all possible content
                          "regrig"     : "regrig/"     ,  //  |  subpackage names
                          "junk"       : "junk/"       ,  //  |  and their base path
                          "folderstore": "folderstore/",  //  |
                          "sidebar"    : "sidebar/"       // -+
                         };
const gksLocaleDir     = "locale/";
const gkoLocalesPacks  = {"en-US":    true,  // the actual value is irrelevant
                          "de-AT":    true,  // all possible locale names for install
                          "de-DE":    true,  // dialog language and content language;
                          "fr-FR":    true,
                          "it-IT":    true,
                          "pl-PL":    true
                         };
const gksLangDefault   = "en-US";            // must be member of gkoLocalesPacks
// argument names
const gksArgLang       = "lang";
const gksArgContent    = "content";
const gksArgLocale     = "locale";
const gksArgOverwrite  = "overwrite";
// file names
const gksInstallProp   = "/install.properties";
const gksInstallLocale = "install/";
const gksJarPath       = "chrome/";
const gksJar           = "mnenhy.jar";
const gksLocaleJarName = "mnenhy.";
const gksLocaleJarExt  = ".jar";
// files and folders
const gkoProChrome     = getFolder("Profile", "chrome");
const gkoProChromeJar  = getFolder(gkoProChrome, gksJar);
const gkbProJarExists  = File.exists(gkoProChromeJar);
const gkoAppChrome     = getFolder("Chrome");
const gkoAppChromeJar  = getFolder(gkoAppChrome, gksJar);
const gkbAppJarExists  = File.exists(gkoAppChromeJar);
// string property names
const gksConfirmLoc    = "Location";
const gksAlertSuccess  = "Success";
const gksAlertFailure  = "Failure";
const gksAlertOldInst  = "OldInst";

// Every once in a while, some JS guys "fix" branch also,
// making eval crash and SM 1.x die when installing Mnenhy. :-(
// I'll define a variable here, so that I don't need to rewrite this whole
// script each time...
const gkbEvalBroken    = true;

// |
// |  variables
// |
var grsProperties      = null;    // array of localized string
var grsInstallContent  = [];      // array of content names
var grsInstallLocales  = [];      // array of locale names
var gbOverwrite        = false;   // ignore old content?

// |
// |  helper functions
// |

// prepare localized install text
//
// since it is even impossible to use String functions like
// indexOf or replace, we can only have static text
// Install.loadResources is pretty useless that way... :(
// -> I'm told I could use new String(), but I didn't change
//    anything here yet...
function Init()
{
  var i,s;
  var sLang = gksLangDefault;
  var oArguments = null;
  if (!gkbEvalBroken)
    oArguments = eval(decodeURIComponent(Install.arguments));
  if (oArguments instanceof Array)
  {
    for (i in oArguments)   // loop over all arguments
    {
      var oPackInfo = oArguments[i];
      if (oPackInfo instanceof Object)
      {
        for (s in oPackInfo)  // loop over all install info
        {
          var sValue = oPackInfo[s];
          switch(s)
          {
            case gksArgOverwrite:
              gbOverwrite = (sValue == true);
              break;
            case gksArgLang:
              if (sValue in gkoLocalesPacks)
                sLang = sValue;
              break;
            case gksArgContent:
              if (sValue in gkoContentPacks)
                grsInstallContent.push(sValue);
              break;
            case gksArgLocale:
              if (sValue in gkoLocalesPacks)
                grsInstallLocales.push(sValue);
              break;
          } // switch
        } // for all packinfo
      } // have packinfo
    } // for all args
  } // have args

  // set defaults
  const kbNoContent = !grsInstallContent.length;
  const kbNoLocales = !grsInstallLocales.length;

  if (kbNoContent && kbNoLocales)
  { // install everything
    for (s in gkoContentPacks)
      grsInstallContent.push(s);
    for (s in gkoLocalesPacks)
      grsInstallLocales.push(s);
  }
  else
  { // just install certain packages
    if (!kbNoContent)
    { // have content, so add base content package
      grsInstallContent.push(gksChromeName);
    }
  }

  logComment("Installing this content: "    + grsInstallContent);
  logComment("Installing these locales: "   + grsInstallLocales);
  logComment("Using this dialog language: " + sLang);
  logComment("Overwrite old installation: " + gbOverwrite);

  grsProperties = loadResources(gksInstallLocale + sLang + gksInstallProp);
  return !!grsProperties;
}


// register locale
function RegisterLocale(asLocale, aiLocation, aoChromePath)
{ // copy the jar
  const ksLocName = gksName + " (" + asLocale + ")";
  const ksJarName = gksLocaleJarName + asLocale + gksLocaleJarExt;
  var iError = addFile(ksLocName, gksVersion, gksJarPath + ksJarName, aoChromePath, null);
  if (iError == SUCCESS)
  { // register the chrome parts
    const oJarPath = getFolder(aoChromePath, ksJarName);
    const ksPath   = gksLocaleDir + asLocale + "/" + gksChromeName + "/";
    registerChrome(LOCALE | aiLocation,   // app or profile
                   oJarPath,              // the locale jar
                   ksPath);               // locale/en-US/mnenhy/
  }
  return iError;
}


// main()
function InstallAddon()
{
  var iError = initInstall(gksName, gksChromeName, gksVersion);
  if (iError == SUCCESS)
  { // load localized properties file
    if (Init())
    {
      var bHaveContent = gkbProJarExists || gkbAppJarExists;
      var oChromeDir   = gkbProJarExists ? gkoProChrome   : gkoAppChrome;
      var iChrome      = gkbProJarExists ? PROFILE_CHROME : DELAYED_CHROME;
      if (grsInstallContent.length)
      { // where do we wanna install today?
        //   if we have already an installation -> cancel with uninstall request
        //                                         (if not overridden)
        //   otherwise                          -> ask where to install it
        //                                         (or use old, if overridden)
        // (an installation both in the app chrome and the profile chrome isn't
        //  possible with this install script, so we can ignore that case)
        // in any case, tell the user what we've found
        var bOverwrite = false;
        if (bHaveContent)
        {
          if (gbOverwrite)
          {
            bOverwrite   = true;
            bHaveContent = false;
          }
          else
            iError = ALREADY_EXISTS;
        }

        if (!bHaveContent)
        { // ask where to install (if not overwritten)
          if (bOverwrite)
            bProfile = gkbProJarExists; // in doubt, use profile
          else
            bProfile = confirm(grsProperties[gksConfirmLoc]);

          // copy the jar
          oChromeDir = bProfile ? gkoProChrome : gkoAppChrome;
          iError = addFile(gksName, gksVersion, gksJarPath + gksJar, oChromeDir, null);
          if (iError == SUCCESS)
          { // register the chrome parts
            iChrome  = bProfile ? PROFILE_CHROME : DELAYED_CHROME;
            const koJarPath = bProfile ? gkoProChromeJar : gkoAppChromeJar;
            // register the content packs
            for (var sCon in grsInstallContent)
            {
              var sPath = gksContentDir + gksChromeName + "/" + gkoContentPacks[grsInstallContent[sCon]];
              registerChrome(CONTENT | iChrome,  // app or profile
                             koJarPath,          // the main jar
                             sPath);             // content/mnenhy/headers/
            }
            bHaveContent = true;
          } // register content
        }
      } // install content

      if ((iError == SUCCESS) && grsInstallLocales.length)
      { // no error yet, but locales ;-)
        // (if we have an error, it is sustained;
        //  not having any locales is fine also)
        if (bHaveContent)
        { // install locales into content location
          // every locale has its own jar file
          for (var sLoc in grsInstallLocales)
          {
            iError = RegisterLocale(grsInstallLocales[sLoc], iChrome, oChromeDir);
            if (iError != SUCCESS)
              break;
          }
        }
        else
        { // do not install locales without having content
          iError = DOES_NOT_EXIST;
        }
      } // have locales

      if (iError == SUCCESS)
        iError = performInstall(); // just do it!
    }
    else
    { // init failed
      iError = SOURCE_DOES_NOT_EXIST;
    }
  } // init install

  if (iError == SUCCESS || iError == REBOOT_NEEDED)
  {
    alert(grsProperties[gksAlertSuccess]);
  }
  else
  {
    if (iError == ALREADY_EXISTS)
      alert(grsProperties[gksAlertOldInst]);
    else
      alert(grsProperties[gksAlertFailure] + iError);
    cancelInstall(iError);
  }
}


// |
// |  start this installation script
// |

// but first warn if we're a testing build
/*
if (!confirm("Mnenhy " + gksVersion + "\n\nis for TESTING PURPOSES ONLY!\n\n\n"
      + "You may want to backup your profile before\ninstalling - you have been warned... ;-)\n\n\n"
      + "Fixes contained in this build:\n\n"
      +  ".10000:\n"
      +  "*   X-Faces now pick up transparent backgrounds\n"
      +  ".10001:\n"
      +  "*   folderstorage startup fix\n"
      +  ".10002:\n"
      +  "*   fixed junk stats sorting\n"
      +  "*   show all from addresses\n"
      +  ".10003:\n"
      +  "*   changed update URL\n"
      +  ".10004:\n"
      +  "*   disable user-defined install for now\n"
      + "\n\n"))
  cancelInstall(INSTALL_CANCELLED);
else
*/
  InstallAddon();
